@@ -7,7 +7,9 @@ vocabs (be they SOAP, REST, XML-RPC or something else).
 
 The basic idea is that we have skos providers. Each provider is an instance
 of a :class:`VocabularyProvider`. The same class can thus be reused with
-different configurations to handle different vocabs.
+different configurations to handle different vocabs. Generally speaking, every
+instance of a certain :class:`VocabularyProvider` will deal with concepts and
+collections from a single conceptscheme.
 '''
 
 from __future__ import unicode_literals
@@ -18,10 +20,14 @@ import warnings
 
 from .skos import (
     Concept,
-    Collection
+    Collection,
+    ConceptScheme
 )
 
-from .uri import DefaultUrnGenerator
+from .uri import (
+    DefaultUrnGenerator,
+    DefaultConceptSchemeUrnGenerator
+)
 
 
 class VocabularyProvider:
@@ -34,15 +40,33 @@ class VocabularyProvider:
         '''Create a new provider and register some metadata.
 
         Expected metadata:
-         * id: A unique identifier for the vocabulary. Required.
-         * default_language: Used to determine what language to use when
-           returning labels if no language is specified.
+         * `id`: A unique identifier for the vocabulary. Required.
+         * `default_language`: Used to determine what language to use when
+           returning labels if no language is specified. Will default to `en`
+           if not specified.
+         * `subject`: A list of subjects or tags that define what the provider
+           is about or what the provider can handle.
+
+        :param uri_generator: An object that implements the 
+            :class:`skosprovider.uri.UriGenerator` interface.
+        :param concept_scheme: A :class:`~skosprovider.skos.ConceptScheme`.
+        :param dict metadata: Metadata essential to this provider.
         '''
+        if not 'subject' in metadata:
+            metadata['subject'] = []
         self.metadata = metadata
         if 'uri_generator' in kwargs:
             self.uri_generator = kwargs.get('uri_generator')
         else:
             self.uri_generator = DefaultUrnGenerator(self.metadata.get('id'))
+        if 'concept_scheme' in kwargs:
+            self.concept_scheme = kwargs.get('concept_scheme')
+        else:
+            self.concept_scheme = ConceptScheme(
+                uri=DefaultConceptSchemeUrnGenerator().generate(
+                    id=self.metadata.get('id')
+                )
+            )
 
     def _get_language(self, **kwargs):
         '''Determine what language to render labels in.
@@ -380,7 +404,7 @@ class MemoryProvider(VocabularyProvider):
         language = self._get_language(**kwargs)
         ret = []
         for c in self.list:
-            if isinstance(c, Concept) and len(c.broader) == 00:
+            if isinstance(c, Concept) and len(c.broader) == 0:
                 ret.append({
                     'id': c.id,
                     'uri': c.uri,
@@ -395,9 +419,8 @@ class MemoryProvider(VocabularyProvider):
             if str(c.id) == id:
                 if isinstance(c, Concept):
                     ret = set([c.id])
-                    if 'narrower' in c:
-                        for cid in c['narrower']:
-                            ret |= set(self.expand(cid))
+                    for cid in c.narrower:
+                        ret |= set(self.expand(cid))
                     return list(ret)
                 elif isinstance(c, Collection):
                     ret = set([])
@@ -407,7 +430,25 @@ class MemoryProvider(VocabularyProvider):
         return False
 
     def get_top_display(self, **kwargs):
-        return self.get_top_concepts(**kwargs)
+        language = self._get_language(**kwargs)
+        ret = []
+        for c in self.list:
+            if isinstance(c, Concept) and len(c.broader) == 0 and len(c.member_of) == 0:
+                ret.append({
+                    'id': c.id,
+                    'uri': c.uri,
+                    'type': c.type,
+                    'label': c.label(language).label
+                })
+            if isinstance(c, Collection) and len(c.superordinates) == 0 and len(c.member_of) == 0:
+                ret.append({
+                    'id': c.id,
+                    'uri': c.uri,
+                    'type': c.type,
+                    'label': c.label(language).label
+                })
+            
+        return ret
 
     def get_children_display(self, id, **kwargs):
         c = self.get_by_id(id)
@@ -416,7 +457,10 @@ class MemoryProvider(VocabularyProvider):
         language = self._get_language(**kwargs)
         ret = []
         if isinstance(c, Concept):
-            display_children = c.narrower
+            if len(c.subordinate_arrays) == 0:
+                display_children = c.narrower
+            else:
+                display_children = c.subordinate_arrays
         else:
             display_children = c.members
         for id in display_children:
@@ -446,20 +490,25 @@ class DictionaryProvider(MemoryProvider):
             return Collection(
                 id=data['id'],
                 uri=data['uri'] if 'uri' in data else self.uri_generator.generate(type='collection', id=data['id']),
+                concept_scheme=self.concept_scheme,
                 labels=data['labels'] if 'labels' in data else [],
                 members=data['members'] if 'members' in data else [],
-                member_of=data['member_of'] if 'member_of' in data else []
+                member_of=data['member_of'] if 'member_of' in data else [],
+                superordinates=data['superordinates'] if 'superordinates' in data else []
             )
         else:
             return Concept(
                 id=data['id'],
                 uri=data['uri'] if 'uri' in data else self.uri_generator.generate(type='concept', id=data['id']),
+                concept_scheme=self.concept_scheme,
                 labels=data['labels'] if 'labels' in data else [],
                 notes=data['notes'] if 'notes' in data else [],
                 broader=data['broader'] if 'broader' in data else [],
                 narrower=data['narrower'] if 'narrower' in data else [],
                 related=data['related'] if 'related' in data else [],
-                member_of=data['member_of'] if 'member_of' in data else []
+                member_of=data['member_of'] if 'member_of' in data else [],
+                subordinate_arrays=data['subordinate_arrays'] if 'subordinate_arrays' in data else [],
+                matches=data['matches'] if 'matches' in data else {}
             )
 
 
