@@ -1,14 +1,18 @@
-# -*- coding: utf-8 -*-
+#-*- coding: utf-8 -*-
 
 '''
-This module contains a read-only model of the :term:`SKOS` specification.
+This module contains a read-only model of the :term:`SKOS` specification. 
+
+To complement the :term:`SKOS` specification, some elements were borrowed
+from the :term:`SKOS-THES` specification (eg. superordinate and 
+subordinate array).
 
 .. versionadded:: 0.2.0
 '''
 
 from __future__ import unicode_literals
 
-import collections
+from language_tags import tags
 
 
 class Label:
@@ -119,17 +123,22 @@ class ConceptScheme:
 
     :param string uri: A :term:`URI` for this conceptscheme.
     :param list labels: A list of :class:`skosprovider.skos.Label` instances.
+    :param list notes: A list of :class:`skosprovider.skos.Note` instances.
     '''
 
     uri = None
     '''A :term:`URI` for this conceptscheme.'''
 
     labels = []
-    '''A list of labels for this conceptscheme.'''
+    '''A :class:`lst` of :class:`skosprovider.skos.label` instances.'''
 
-    def __init__(self, uri, labels=[]):
+    notes = []
+    '''A :class:`lst` of :class:`skosprovider.skos.Note` instances.'''
+
+    def __init__(self, uri, labels=[], notes=[]):
         self.uri = uri
-        self.labels = labels
+        self.labels = [dict_to_label(l) for l in labels]
+        self.notes = [dict_to_note(n) for n in notes]
 
     def label(self, language='any'):
         '''
@@ -138,13 +147,14 @@ class ConceptScheme:
         This uses the :func:`label` function to determine which label to
         return.
 
-        :param string language: The preferred language to receive the label in.
-        :rtype: :class:`skosprovider.skos.Label` or False if no labels was found.
+        :param string language: The preferred language to receive the label in. 
+            This should be a valid IANA language tag.
+        :rtype: :class:`skosprovider.skos.Label` or False if no labels were found.
         '''
         return label(self.labels, language)
 
 
-class Concept(collections.Mapping):
+class Concept:
     '''
     A :term:`SKOS` Concept.
     '''
@@ -167,29 +177,63 @@ class Concept(collections.Mapping):
     eg. 'concept'
     '''
 
+    concept_scheme = None
+    '''The :class:`ConceptScheme` this Concept is a part of.'''
+
+    labels = []
+    '''A :class:`lst` of :class:`Label` instances.'''
+
+    notes = []
+    '''A :class:`lst` of :class:`Note` instances.'''
+
+    broader = []
+    '''A :class:`lst` of concept ids.'''
+
+    narrower = []
+    '''A :class:`lst` of concept ids.'''
+
+    related = []
+    '''A :class:`lst` of concept ids.'''
+
+    member_of = []
+    '''A :class:`lst` of collection ids.'''
+
+    subordinate_arrays = []
+    '''A :class:`list` of collection ids.'''
+
+    matches = {
+        'close': [],
+        'exact': [],
+        'related': [],
+        'broader': [],
+        'narrower': []
+    }
+    '''Matches with Concepts in other ConceptSchemes.
+
+    This dictionary contains a key for each type of Match (close, exact, 
+    related, broader, narrower). Attached to each key is a list of URI's.
+    '''
+
     def __init__(self, id, uri=None,
+                 concept_scheme=None,
                  labels=[], notes=[],
                  broader=[], narrower=[], related=[],
-                 member_of=[]):
+                 member_of=[], subordinate_arrays=[],
+                 matches={}):
         self.id = id
         self.uri = uri
         self.type = 'concept'
+        self.concept_scheme = concept_scheme
         self.labels = [dict_to_label(l) for l in labels]
         self.notes = [dict_to_note(n) for n in notes]
         self.broader = broader
         self.narrower = narrower
         self.related = related
         self.member_of = member_of
-
-    def __getitem__(self, item):
-        if item in self.__dict__.keys():
-            return self.__dict__[item]
-
-    def __iter__(self):
-        return iter(self.__dict__)
-
-    def __len__(self):
-        return len(self.__dict__)
+        self.subordinate_arrays = subordinate_arrays
+        for match_type in matches.keys():
+            if match_type in self.matches.keys():
+                self.matches[match_type] = matches.get(match_type, [])
 
     def label(self, language='any'):
         '''
@@ -197,8 +241,9 @@ class Concept(collections.Mapping):
 
         This uses the :func:`label` function to determine which label to return.
 
-        :param string language: The preferred language to receive the label in.
-        :rtype: :class:`skosprovider.skos.Label` or False if no labels was found.
+        :param string language: The preferred language to receive the label in. 
+            This should be a valid IANA language tag.
+        :rtype: :class:`skosprovider.skos.Label` or False if no labels were found.
         '''
         return label(self.labels, language)
 
@@ -220,22 +265,38 @@ class Collection:
     eg. 'collection'
     '''
 
+    concept_scheme = None
+    '''The :class:`ConceptScheme` this Collection is a part of.'''
+
     labels = []
     '''A :class:`lst` of :class:`skosprovider.skos.label` instances.'''
 
-    members = []
-    '''A :class:`lst` of concept ids.'''
+    notes = []
+    '''A :class:`lst` of :class:`skosprovider.skos.Note` instances.'''
 
-    member_of = []
+    members = []
     '''A :class:`lst` of concept or collection ids.'''
 
-    def __init__(self, id, uri=None, labels=[], members=[], member_of=[]):
+    member_of = []
+    '''A :class:`lst` of collection ids.'''
+
+    superordinates = []
+    '''A :class:`lst` of concept ids.'''
+
+    def __init__(self, id, uri=None,
+                 concept_scheme=None,
+                 labels=[], notes=[], 
+                 members=[], member_of=[],
+                 superordinates=[]):
         self.id = id
         self.uri = uri
         self.type = 'collection'
+        self.concept_scheme = concept_scheme
         self.labels = [dict_to_label(l) for l in labels]
+        self.notes = [dict_to_note(n) for n in notes]
         self.members = members
         self.member_of = member_of
+        self.superordinates = superordinates
 
     def label(self, language='any'):
         '''
@@ -243,8 +304,9 @@ class Collection:
 
         This uses the :func:`label` function to determine which label to return.
 
-        :param string language: The preferred language to receive the label in.µ
-        :rtype: :class:`skosprovider.skos.Label` or False if no labels was found.
+        :param string language: The preferred language to receive the label in. 
+            This should be a valid IANA language tag.
+        :rtype: :class:`skosprovider.skos.Label` or False if no labels were found.
         '''
         return label(self.labels, language)
 
@@ -261,6 +323,14 @@ def label(labels=[], language='any'):
     a pref label for the specified language. If there's no pref label,
     it looks for an alt label. It disregards hidden labels.
 
+    While matching languages, preference will be given to exact matches. But,
+    if no exact match is present, an inexact match will be attempted. This might
+    be because a label in language `nl-BE` is being requested, but only `nl` or
+    even `nl-NL` is present. Similarly, when requesting `nl`, a label with
+    language `nl-NL` or even `nl-Latn-NL` will also be considered, 
+    providing no label is present that has an exact match with the 
+    requested language.
+
     If language 'any' was specified, all labels will be considered,
     regardless of language.
 
@@ -270,21 +340,34 @@ def label(labels=[], language='any'):
     method will automatically try to find a label in some other language.
 
     Finally, if no label could be found, None is returned.
+
+    :param string language: The preferred language to receive the label in. This
+        should be a valid IANA language tag.
     '''
+    # Normalise the tag
+    broader_language_tag = None
+    if language != 'any':
+        language = tags.tag(language).format
+        broader_language_tag = tags.tag(language).language
+    pref = None
     alt = None
     for l in labels:
         l = dict_to_label(l)
         if language == 'any' or l.language == language:
-            if l.type == 'prefLabel':
-                return l
-            if alt is None and l.type == 'altLabel':
+            if l.type == 'prefLabel' and (pref is None or pref.language != language):
+                pref = l
+            if l.type == 'altLabel' and (alt is None or alt.language != language):
                 alt = l
-    if alt is not None:
+        if broader_language_tag and tags.tag(l.language).language and tags.tag(l.language).language.format == broader_language_tag.format:
+            if l.type == 'prefLabel' and pref is None:
+                pref = l
+            if l.type == 'altLabel' and alt is None:
+                alt = l
+    if pref is not None:
+        return pref
+    elif alt is not None:
         return alt
-    elif language != 'any':
-        return label(labels, 'any')
-    else:
-        return None
+    return label(labels, 'any') if language != 'any' else None
 
 
 def dict_to_label(dict):
