@@ -3,6 +3,8 @@
 This module contains utility functions for dealing with skos providers.
 '''
 
+from __future__ import unicode_literals
+
 from skosprovider.skos import (
     Concept,
     Collection
@@ -24,22 +26,28 @@ def dict_dumper(provider):
     for stuff in provider.get_all():
         c = provider.get_by_id(stuff['id'])
         labels = [l.__dict__ for l in c.labels]
+        notes = [n.__dict__ for n in c.notes]
         if isinstance(c, Concept):
-            notes = [n.__dict__ for n in c.notes]
             ret.append({
                 'id': c.id,
-                'type': 'concept',
+                'uri': c.uri,
+                'type': c.type,
                 'labels': labels,
                 'notes': notes,
                 'narrower': c.narrower,
                 'broader': c.broader,
-                'related': c.related
+                'related': c.related,
+                'member_of': c.member_of
             })
         elif isinstance(c, Collection):
             ret.append({
                 'id': c.id,
-                'type': 'collection',
+                'uri': c.uri,
+                'type': c.type,
                 'labels': labels,
-                'members': c.members
+                'notes': notes,
+                'members': c.members,
+                'member_of': c.member_of,
+                'superordinates': c.superordinates
             })
     return ret
